@@ -339,6 +339,16 @@ const std::map<uint32_t, Altera::max10_mem_t> Altera::max10_memory_map = {
 		.done_bit_addr = 0x0011,  // done bit
 		.pgm_success_addr = 0x0015}  // program success addr
 	},
+	{0x031840dd, { // 10M25SA
+		.check_addr0 = 0x80009, // check_addr0
+		.dsm_addr = 0x0000, .dsm_len = 1024, // DSM
+		.ufm_addr = 0x0400, .ufm_len = {4096, 4096}, // UFM
+		.cfm_addr = 0x2400, .cfm_len = {94208, 40960, 53248}, // CFM
+		.sectors_erase_addr = {0x17ffff, 0x27ffff, 0x37ffff, 0x47ffff, 0x57ffff}, // sectors erase address
+		.done_bit_addr = 0x0015, // done bit
+		// CHECKME: this line
+		.pgm_success_addr = 0x0015} // program success addr
+	},
 };
 
 /* Write an arbitrary file in UFM1, UFM0 by default and also CFM2 and CFM1 if
