@@ -33,6 +33,7 @@ class CH347Jtag : public JtagInterface {
  private:
 	bool _verbose;
 	bool _is_largerPack;
+	int _jtagIntf;
 	int setClk(const uint8_t &factor);
 
 	libusb_device_handle *dev_handle;
@@ -43,4 +44,7 @@ class CH347Jtag : public JtagInterface {
 	uint8_t *obuf;
 	int get_obuf_length() const {return MAX_BUFFER - (obuf - _obuf);}
 	int usb_xfer(unsigned wlen, unsigned rlen, unsigned *actual, bool defer);
+
+	uint8_t _tdi;
+	uint8_t _tms;
 };
