@@ -13,29 +13,55 @@
 #include <sstream>
 #include <vector>
 
+#ifdef ENABLE_ALTERA_SUPPORT
 #include "altera.hpp"
+#endif
+#ifdef ENABLE_ANLOGIC_SUPPORT
 #include "anlogic.hpp"
+#endif
 #include "board.hpp"
 #include "cable.hpp"
+#ifdef ENABLE_COLOGNECHIP_SUPPORT
 #include "colognechip.hpp"
+#endif
 #include "common.hpp"
 #include "cxxopts.hpp"
 #include "device.hpp"
+#ifdef ENABLE_DFU
 #include "dfu.hpp"
+#endif
 #include "display.hpp"
+#ifdef ENABLE_EFINIX_SUPPORT
 #include "efinix.hpp"
+#endif
+#ifdef USE_LIBFTDI
 #include "ftdispi.hpp"
+#endif
+#ifdef ENABLE_GOWIN_SUPPORT
 #include "gowin.hpp"
-#include "ice40.hpp"
+#endif
+#ifdef ENABLE_LATTICE_SUPPORT
 #include "lattice.hpp"
+#endif
+#ifdef ENABLE_ICE40_SUPPORT
+#include "ice40.hpp"
+#endif
+#ifdef ENABLE_LATTICESSPI_SUPPORT
 #include "latticeSSPI.hpp"
+#endif
+#ifdef ENABLE_USB_SCAN
 #include "libusb_ll.hpp"
+#endif
 #include "jtag.hpp"
 #include "part.hpp"
 #include "spiFlash.hpp"
 #include "rawParser.hpp"
+#ifdef ENABLE_XILINX_SUPPORT
 #include "xilinx.hpp"
+#endif
+#ifdef ENABLE_SVF_JTAG
 #include "svf_jtag.hpp"
+#endif
 #ifdef ENABLE_XVC
 #include "xvc_server.hpp"
 #endif
@@ -74,6 +100,7 @@ struct arguments {
 	unsigned int file_size;
 	string target_flash;
 	bool external_flash;
+	bool spi_flash_type;
 	int16_t altsetting;
 	uint16_t vid;
 	uint16_t pid;
@@ -124,8 +151,8 @@ int main(int argc, char **argv)
 			-1, 0, false, "-", false, false, false, false, Device::PRG_NONE, false,
 			/* spi dfu    file_type fpga_part bridge_path probe_firmware */
 			false, false, "",       "",       "",         "",
-			/* index_chain file_size target_flash external_flash altsetting */
-			-1,            0,        "primary",   false,         -1,
+			/* index_chain file_size target_flash external_flash spi_flash_type altsetting */
+			-1,            0,        "primary",   false,         true,          -1,
 			/* vid, pid, index bus_addr, device_addr */
 				0,   0,   -1,     0,         0,
 			"127.0.0.1", 0, false, false, false, false, "", false, false,
@@ -197,6 +224,7 @@ int main(int argc, char **argv)
 		 */
 		if (args.freq == 0)
 			args.freq = board->default_freq;
+		args.spi_flash_type = board->spi_bpi == SPI_FLASH;
 	}
 
 	if (args.cable[0] == '-') { /* if no board and no cable */
@@ -217,6 +245,7 @@ int main(int argc, char **argv)
 	}
 	cable = select_cable->second;
 
+#ifdef USE_LIBFTDI
 	if (args.ftdi_channel != -1) {
 		if (cable.type != MODE_FTDI_SERIAL && cable.type != MODE_FTDI_BITBANG){
 			printError("Error: FTDI channel param is for FTDI cables.");
@@ -241,6 +270,7 @@ int main(int argc, char **argv)
 			return EXIT_FAILURE;
 		}
 	}
+#endif
 
 	if (args.vid != 0) {
 		printInfo("Cable VID overridden");
@@ -258,6 +288,7 @@ int main(int argc, char **argv)
 	cable.config.index = args.cable_index;
 	cable.config.status_pin = args.status_pin;
 
+#ifdef USE_LIBFTDI
 	/* FLASH direct access */
 	if (args.spi || (board && board->mode == COMM_SPI)) {
 		/* if no instruction from user -> select flash mode */
@@ -291,25 +322,45 @@ int main(int argc, char **argv)
 		if (board && board->manufacturer != "none") {
 			Device *target;
 			if (board->manufacturer == "efinix") {
+#ifdef ENABLE_EFINIX_SUPPORT
 				target = new Efinix(spi, args.bit_file, args.file_type,
 					board->reset_pin, board->done_pin, board->oe_pin,
 					args.verify, args.verbose);
+#else
+				printError("Support for Efinix FPGAs was not enabled at compile time");
+				return EXIT_FAILURE;
+#endif
 			} else if (board->manufacturer == "lattice") {
 				if (board->fpga_part == "ice40") {
+#ifdef ENABLE_ICE40_SUPPORT
 					target = new Ice40(spi, args.bit_file, args.file_type,
 						args.prg_type,
 						board->reset_pin, board->done_pin, args.verify, args.verbose);
+#else
+					printError("Support for ICE40 FPGAs was not enabled at compile time");
+					return EXIT_FAILURE;
+#endif
 				} else if (board->fpga_part == "ecp5") {
+#ifdef ENABLE_LATTICESSPI_SUPPORT
 					target = new LatticeSSPI(spi, args.bit_file, args.file_type, args.verbose);
+#else
+					printError("Support for Lattice FPGAs (SSPI mode) was not enabled at compile time");
+					return EXIT_FAILURE;
+#endif
 				} else {
 					printError("Error (SPI mode): " + board->fpga_part +
 						" is an unsupported/unknown Lattice Model");
 					return EXIT_FAILURE;
 				}
 			} else if (board->manufacturer == "colognechip") {
+#ifdef ENABLE_COLOGNECHIP_SUPPORT
 				target = new CologneChip(spi, args.bit_file, args.file_type, args.prg_type,
 					board->reset_pin, board->done_pin, DBUS6, board->oe_pin,
 					args.verify, args.verbose);
+#else
+				printError("Support for Gowin FPGAs was not enabled at compile time");
+				return EXIT_FAILURE;
+#endif
 			} else {
 				printError("Error (SPI mode): " + board->manufacturer +
 					" is an unsupported/unknown target");
@@ -404,11 +455,13 @@ int main(int argc, char **argv)
 
 		return spi_ret;
 	}
+#endif
 
 	/* ------------------- */
 	/* DFU access          */
 	/* ------------------- */
 	if (args.dfu || (board && board->mode == COMM_DFU)) {
+#ifdef ENABLE_DFU
 		/* try to init DFU probe */
 		DFU *dfu = NULL;
 		uint16_t vid = 0, pid = 0;
@@ -458,6 +511,10 @@ int main(int argc, char **argv)
 		}
 
 		return EXIT_SUCCESS;
+#else
+		printError("DFU support: disabled at build time");
+		return EXIT_FAILURE;
+#endif
 	}
 
 #ifdef ENABLE_XVC
@@ -569,6 +626,7 @@ int main(int argc, char **argv)
 	/* detect svf file and program the device */
 	if (!args.file_type.compare("svf") ||
 			args.bit_file.find(".svf") != string::npos) {
+#ifdef ENABLE_SVF_JTAG
 		SVF_jtag *svf = new SVF_jtag(jtag, args.verbose);
 		try {
 			svf->parse(args.bit_file);
@@ -576,6 +634,11 @@ int main(int argc, char **argv)
 			return EXIT_FAILURE;
 		}
 		return EXIT_SUCCESS;
+#else
+		printError("Support for SVF Jtag was not enabled at compile time");
+		delete(jtag);
+		return EXIT_FAILURE;
+#endif
 	}
 
 	/* check if selected device is supported
@@ -593,30 +656,72 @@ int main(int argc, char **argv)
 	Device *fpga;
 	try {
 		if (fab == "xilinx") {
+#ifdef ENABLE_XILINX_SUPPORT
 			fpga = new Xilinx(jtag, args.bit_file, args.secondary_bit_file,
-				args.file_type, args.prg_type, args.fpga_part, args.bridge_path,
+				args.file_type, args.prg_type, args.fpga_part, args.spi_flash_type, args.bridge_path,
 				args.target_flash, args.verify, args.verbose, args.skip_load_bridge, args.skip_reset,
 				args.read_dna, args.read_xadc);
+#else
+			printError("Support for Xilinx FPGAs was not enabled at compile time");
+			delete(jtag);
+			return EXIT_FAILURE;
+#endif
 		} else if (fab == "altera") {
+#ifdef ENABLE_ALTERA_SUPPORT
 			fpga = new Altera(jtag, args.bit_file, args.file_type,
 				args.prg_type, args.fpga_part, args.bridge_path, args.verify,
 				args.verbose, args.flash_sector, args.skip_load_bridge, args.skip_reset);
+#else
+			printError("Support for Altera FPGAs was not enabled at compile time");
+			delete(jtag);
+			return EXIT_FAILURE;
+#endif
 		} else if (fab == "anlogic") {
+#ifdef ENABLE_ANLOGIC_SUPPORT
 			fpga = new Anlogic(jtag, args.bit_file, args.file_type,
 				args.prg_type, args.verify, args.verbose);
+#else
+			printError("Support for Anlogic FPGAs was not enabled at compile time");
+			delete(jtag);
+			return EXIT_FAILURE;
+#endif
 		} else if (fab == "efinix") {
+#ifdef ENABLE_EFINIX_SUPPORT
 			fpga = new Efinix(jtag, args.bit_file, args.file_type,
 				args.prg_type, args.board, args.fpga_part, args.bridge_path,
 				args.verify, args.verbose);
+#else
+			printError("Support for Efinix FPGAs was not enabled at compile time");
+			delete(jtag);
+			return EXIT_FAILURE;
+#endif
 		} else if (fab == "Gowin") {
+#ifdef ENABLE_GOWIN_SUPPORT
 			fpga = new Gowin(jtag, args.bit_file, args.file_type, args.mcufw,
 				args.prg_type, args.external_flash, args.verify, args.verbose, args.user_flash);
+#else
+			printError("Support for Gowin FPGAs was not enabled at compile time");
+			delete(jtag);
+			return EXIT_FAILURE;
+#endif
 		} else if (fab == "lattice") {
+#ifdef ENABLE_LATTICE_SUPPORT
 			fpga = new Lattice(jtag, args.bit_file, args.file_type,
 				args.prg_type, args.flash_sector, args.verify, args.verbose, args.skip_load_bridge, args.skip_reset);
+#else
+			printError("Support for Lattice FPGAs was not enabled at compile time");
+			delete(jtag);
+			return EXIT_FAILURE;
+#endif
 		} else if (fab == "colognechip") {
+#ifdef ENABLE_COLOGNECHIP_SUPPORT
 			fpga = new CologneChip(jtag, args.bit_file, args.file_type,
 				args.prg_type, args.board, args.cable, args.verify, args.verbose);
+#else
+			printError("Support for Gowin FPGAs was not enabled at compile time");
+			delete(jtag);
+			return EXIT_FAILURE;
+#endif
 		} else {
 			printError("Error: manufacturer " + fab + " not supported");
 			delete(jtag);
@@ -1193,9 +1298,11 @@ void displaySupported(const struct arguments &args)
 		cout << endl;
 	}
 
+#ifdef ENABLE_USB_SCAN
 	if (args.scan_usb) {
 		libusb_ll usb(0, 0, args.verbose);
 		usb.scan();
 	}
+#endif
 }
 
