@@ -150,6 +150,7 @@ static std::map<std::string, std::map<std::string, std::vector<uint8_t>>>
 				{ "JSHUTDOWN",   {0x0D} },
 				{ "ISC_PROGRAM", {0x11} },
 				{ "ISC_DISABLE", {0x16} },
+				{ "STATUS", 	 {0x1F} },
 				{ "BYPASS",      {0xff} },
 			}
 		},
@@ -271,7 +272,9 @@ Xilinx::Xilinx(Jtag *jtag, const std::string &filename,
 	const std::string &secondary_filename,
 	const std::string &file_type,
 	Device::prog_type_t prg_type,
-	const std::string &device_package, const std::string &spiOverJtagPath,
+	const std::string &device_package,
+	const bool spi_flash_type,
+	const std::string &spiOverJtagPath,
 	const std::string &target_flash,
 	bool verify, int8_t verbose,
 	bool skip_load_bridge, bool skip_reset, bool read_dna, bool read_xadc):
@@ -280,7 +283,7 @@ Xilinx::Xilinx(Jtag *jtag, const std::string &filename,
 				 skip_reset),
 	_device_package(device_package), _spiOverJtagPath(spiOverJtagPath),
 	_irlen(6), _secondary_filename(secondary_filename), _soj_is_v2(false),
-	_jtag_chain_len(1)
+	_jtag_chain_len(1), _is_bpi_board(!spi_flash_type)
 {
 	if (prg_type == Device::RD_FLASH) {
 		_mode = Device::READ_MODE;
@@ -294,7 +297,9 @@ Xilinx::Xilinx(Jtag *jtag, const std::string &filename,
 				_mode = Device::SPI_MODE;
 		} else if (_file_extension == "jed") {
 			_mode = Device::FLASH_MODE;
-		} else {
+		} else if (_file_extension == "pdi") {
+			_mode = Device::MEM_MODE;
+		}  else {
 			_mode = Device::SPI_MODE;
 		}
 	}
@@ -358,6 +363,14 @@ Xilinx::Xilinx(Jtag *jtag, const std::string &filename,
 		_fpga_family = KINTEXUSP_FAMILY;
 	} else if (family == "artixusp") {
 		_fpga_family = ARTIXUSP_FAMILY;
+	} else if (family == "spartanusp") {		
+		if (_file_extension != "pdi") {
+			char mess[256];
+			snprintf(mess, 256, "Error: only volatile PDI programing for "
+				"Spartan Ultrascale+ devices\n");
+			throw std::runtime_error(mess);
+		}
+		_fpga_family = SPARTANUSP_FAMILY;
 	} else if (family == "virtexus") {
 		_fpga_family = VIRTEXUS_FAMILY;
 	} else if (family == "virtexusp") {
@@ -365,9 +378,6 @@ Xilinx::Xilinx(Jtag *jtag, const std::string &filename,
 		_ircode_map = ircode_mapping.at("virtexusp");
 	} else if (family.substr(0, 8) == "spartan3") {
 		_fpga_family = SPARTAN3_FAMILY;
-		if (_mode != Device::MEM_MODE) {
-			throw std::runtime_error("Error: Only load to mem is supported");
-		}
 	} else if (family == "xcf") {
 		_fpga_family = XCF_FAMILY;
 		if (_mode == Device::MEM_MODE) {
@@ -612,6 +622,9 @@ void Xilinx::program(unsigned int offset, bool unprotect_flash)
 	if (_mode == Device::MEM_MODE || _fpga_family == XCF_FAMILY)
 		reverse = true;
 
+	if (_file_extension == "pdi")
+		reverse = false;
+
 	try {
 		if (_flash_chips & PRIMARY_FLASH) {
 			open_bitfile(_filename, _file_extension, &bit, reverse, _verbose);
@@ -643,17 +656,22 @@ void Xilinx::program(unsigned int offset, bool unprotect_flash)
 	}
 
 	if (_mode == Device::SPI_MODE) {
-		if (_flash_chips & PRIMARY_FLASH) {
-			select_flash_chip(PRIMARY_FLASH);
-			program_spi(bit, offset, unprotect_flash);
-		}
-		if (_flash_chips & SECONDARY_FLASH) {
-			select_flash_chip(SECONDARY_FLASH);
-			program_spi(secondary_bit, offset, unprotect_flash);
-		}
-
-		reset();
+		/* Check for BPI flash boards */
+		if (_is_bpi_board) {
+			program_bpi(bit, offset);
+			reset();
+		} else {
+			if (_flash_chips & PRIMARY_FLASH) {
+				select_flash_chip(PRIMARY_FLASH);
+				program_spi(bit, _file_extension, offset, unprotect_flash);
+			}
+			if (_flash_chips & SECONDARY_FLASH) {
+				select_flash_chip(SECONDARY_FLASH);
+				program_spi(secondary_bit, _secondary_file_extension, offset, unprotect_flash);
+			}
 
+			reset();
+		}
 	} else {
 		if (_fpga_family == SPARTAN3_FAMILY)
 			xc3s_flow_program(bit);
@@ -715,11 +733,11 @@ bool Xilinx::load_bridge()
 	bitname = PathHelper::absolutePath(bitname);
 #endif
 
-	std::cout << "use: " << bitname << std::endl;
-
 	/* first: load spi over jtag */
 	try {
 		BitParser bridge(bitname, true, _verbose);
+		printSuccess("Use: " + bridge.getFilename());
+
 		bridge.parse();
 		if (_fpga_family == SPARTAN3_FAMILY)
 			xc3s_flow_program(&bridge);
@@ -733,6 +751,68 @@ bool Xilinx::load_bridge()
 	return true;
 }
 
+bool Xilinx::load_bpi_bridge()
+{
+	std::string bitname;
+
+	if (_device_package.empty()) {
+		printError("Can't program BPI flash: missing device-package information");
+		return false;
+	}
+
+	bitname = get_shell_env_var("OPENFPGALOADER_SOJ_DIR", DATA_DIR "/openFPGALoader");
+	bitname += "/bpiOverJtag_" + _device_package + ".bit.gz";
+
+#if defined (_WIN64) || defined (_WIN32)
+	bitname = PathHelper::absolutePath(bitname);
+#endif
+
+	/* Load BPI over JTAG bridge */
+	try {
+		BitParser bridge(bitname, true, _verbose);
+		printSuccess("Use: " + bridge.getFilename());
+
+		bridge.parse();
+		program_mem(&bridge);
+	} catch (std::exception &e) {
+		printError(e.what());
+		throw std::runtime_error(e.what());
+	}
+
+	/* Initialize BPI flash instance */
+	_bpi_flash.reset(new BPIFlash(_jtag, _verbose));
+
+	return true;
+}
+
+void Xilinx::program_bpi(ConfigBitstreamParser *bit, unsigned int offset)
+{
+	if (!bit)
+		throw std::runtime_error("called with null bitstream");
+
+	/* Load BPI bridge if not already loaded */
+	if (!_bpi_flash) {
+		if (!load_bpi_bridge()) {
+			throw std::runtime_error("Failed to load BPI bridge");
+		}
+	}
+
+	/* Detect flash */
+	if (!_bpi_flash->detect()) {
+		throw std::runtime_error("BPI flash detection failed");
+	}
+
+	/* Program the flash */
+	const uint8_t *data = bit->getData();
+	int length = bit->getLength() / 8;
+
+	if (!_bpi_flash->write(offset, data, length)) {
+		throw std::runtime_error("BPI flash programming failed");
+	}
+
+	printInfo("BPI flash programming complete");
+}
+
 float Xilinx::get_spiOverJtag_version()
 {
 	uint8_t jtx[6] = {0x01, 0x00, 0x00, 0x00, 0x00, 0x00};
@@ -755,14 +835,19 @@ float Xilinx::get_spiOverJtag_version()
 	return version;
 }
 
-void Xilinx::program_spi(ConfigBitstreamParser * bit, unsigned int offset,
-		bool unprotect_flash)
+void Xilinx::program_spi(ConfigBitstreamParser * bit, std::string extention,
+		unsigned int offset, bool unprotect_flash)
 {
 	if (!bit)
 		throw std::runtime_error("called with null bitstream");
-	const uint8_t *data = bit->getData();
-	int length = bit->getLength() / 8;
-	SPIInterface::write(offset, data, length, unprotect_flash);
+	if (extention == "mcs") {
+		McsParser *parser = (McsParser *)bit;
+		SPIInterface::write(parser->getRecords(), unprotect_flash, true);
+	} else {
+		const uint8_t *data = bit->getData();
+		int length = bit->getLength() / 8;
+		SPIInterface::write(offset, data, length, unprotect_flash);
+	}
 }
 
 void Xilinx::program_mem(ConfigBitstreamParser *bitfile)
@@ -791,11 +876,11 @@ void Xilinx::program_mem(ConfigBitstreamParser *bitfile)
 	 *    the TLR (Test-Logic-Reset) state.
 	 */
 	_jtag->shiftIR(get_ircode(_ircode_map, "JPROGRAM"), NULL, _irlen);
-	/* test */
+	/* Poll INIT_B (bit 4 of IR capture) until config memory is cleared */
 	tx_buf = get_ircode(_ircode_map, "BYPASS");
 	do {
 		_jtag->shiftIR(tx_buf, rx_buf, _irlen);
-	} while (!(rx_buf[0] &0x01));
+	} while (!(rx_buf[0] & 0x10));
 	/*
 	 * 8: Move into the RTI state.                        X     0   10,000(1)
 	 */
@@ -825,7 +910,7 @@ void Xilinx::program_mem(ConfigBitstreamParser *bitfile)
 	const uint8_t *data = bitfile->getData();
 	int tx_len;
 	Jtag::tapState_t tx_end;
-	int burst_len = byte_length / 100;
+	const int burst_len = (byte_length < 100) ? byte_length : byte_length / 100;
 
 	ProgressBar progress("Load SRAM", byte_length, 50, _quiet);
 
@@ -852,41 +937,60 @@ void Xilinx::program_mem(ConfigBitstreamParser *bitfile)
 	 * 16: Move into RTI state.                           X     0   1
 	 */
 	_jtag->set_state(Jtag::RUN_TEST_IDLE);
-	/*
-	 * 17: Enter the SELECT-IR state.                     X     1   2
-	 * 18: Move to the SHIFT-IR state.                    X     0   2
-	 * 19: Start loading the JSTART instruction
-	 *     (optional). The JSTART instruction           01100   0   5
-	 *     initializes the startup sequence.
-	 * 20: Load the last bit of the JSTART instruction.   0     1   1
-	 * 21: Move to the UPDATE-IR state.                   X     1   1
-	 */
-	_jtag->shiftIR(get_ircode(_ircode_map, "JSTART"), NULL, _irlen, Jtag::UPDATE_IR);
-	/*
-	 * 22: Move to the RTI state and clock the
-	 *     startup sequence by applying a minimum         X     0   2000
-	 *     of 2000 clock cycles to the TCK.
-	 */
-	_jtag->set_state(Jtag::RUN_TEST_IDLE);
-	_jtag->toggleClk(2000);
-	/*
-	 * 23: Move to the TLR state. The device is
-	 * now functional.                                    X     1   3
-	 */
-	_jtag->go_test_logic_reset();
-	/* Some xc7s50 does not detect correct connected flash w/o this shift*/
-	_jtag->shiftIR(tx_buf, rx_buf, _irlen);
-	uint8_t ir_c = rx_buf[0] & 0x03;
-	uint8_t isc_done = ((rx_buf[0] >> 2) & 0x01);
-	uint8_t isc_ena  = ((rx_buf[0] >> 3) & 0x01);
-	uint8_t init     = ((rx_buf[0] >> 4) & 0x01);
-	uint8_t done     = ((rx_buf[0] >> 5) & 0x01);
-	printf("Shift IR %02x\n", rx_buf[0]);
-	printf("ir: %x isc_done %x isc_ena %x init %x done %x\n", ir_c, isc_done, isc_ena,
-		init, done);
 
-	if (!done) {
-		read_register("STAT");
+	if (_file_extension == "pdi") {
+		_jtag->toggleClk(2000);
+		/*
+		* 17: For PDI devices, use the STATUS instruction 
+		*     to verify successful configuration.          
+		*/
+		unsigned char tx_data[6]= {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
+		unsigned char rx_data[6];
+		_jtag->shiftIR(get_ircode(_ircode_map, "STATUS"), NULL, _irlen);
+		_jtag->shiftDR(tx_data, rx_data, 48);
+		if ((rx_data[4] & 0x04) != 0x04) {
+				printError("PDI programing failed");
+			} else {
+				printSuccess("PDI programing success");
+			}
+		}
+	else {
+		/*
+		* 17: Enter the SELECT-IR state.                     X     1   2
+		* 18: Move to the SHIFT-IR state.                    X     0   2
+		* 19: Start loading the JSTART instruction
+		*     (optional). The JSTART instruction           01100   0   5
+		*     initializes the startup sequence.
+		* 20: Load the last bit of the JSTART instruction.   0     1   1
+		* 21: Move to the UPDATE-IR state.                   X     1   1
+		*/
+		_jtag->shiftIR(get_ircode(_ircode_map, "JSTART"), NULL, _irlen, Jtag::UPDATE_IR);
+		/*
+		* 22: Move to the RTI state and clock the
+		*     startup sequence by applying a minimum         X     0   2000
+		*     of 2000 clock cycles to the TCK.
+		*/
+		_jtag->set_state(Jtag::RUN_TEST_IDLE);
+		_jtag->toggleClk(2000);
+		/*
+		* 23: Move to the TLR state. The device is
+		* now functional.                                    X     1   3
+		*/
+		_jtag->go_test_logic_reset();
+		/* Some xc7s50 does not detect correct connected flash w/o this shift*/
+		_jtag->shiftIR(tx_buf, rx_buf, _irlen);
+		uint8_t ir_c = rx_buf[0] & 0x03;
+		uint8_t isc_done = ((rx_buf[0] >> 2) & 0x01);
+		uint8_t isc_ena  = ((rx_buf[0] >> 3) & 0x01);
+		uint8_t init     = ((rx_buf[0] >> 4) & 0x01);
+		uint8_t done     = ((rx_buf[0] >> 5) & 0x01);
+		printf("Shift IR %02x\n", rx_buf[0]);
+		printf("ir: %x isc_done %x isc_ena %x init %x done %x\n", ir_c, isc_done, isc_ena,
+			init, done);
+
+		if (!done) {
+			read_register("STAT");
+		}
 	}
 }
 
